/*
 *  Copyright (c) 1999 by Niels Gorisse <niels@bonneville.nl>
 *  Modifications for WireProvider and Large Sysex (c) 2000
 *  by Gerrit Gehnen <Gerrit.Gehnen@gmx.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/**
 * WireNative is the bridge between Java and C++. All classes in C++
 * are written without using any Java-like specification, except the
 * MidiJav.cpp file.
 * @author    <A HREF="mailto:niels@bonneville.nl">Niels Gorisse</A>, 22 april 1999, Bonneville
 * @version   WireNative 0.98
 **/

package WireProvider;
import javax.sound.midi.MidiUnavailableException;


public final class WireNative
	{
private int port; // -1 represents none
private int Jthis; // referentie van over welk thing we het hebben
private int dir;  //XX

static {
//  System.out.println("WireNeative: Static block");
  try{
    System.loadLibrary("WireNative");
  }
  catch (Exception e) {e.printStackTrace ();}
//  System.out.println("WireNeative: Static block Ende");
   
}
//Direction 0: Input
//Direction 1: Output
public WireNative(int direction) throws Exception
	{
        dir=direction; //XX
//        System.out.println("WireNative: Konstruktor "+direction);
          port=-1;
    Jthis=newJavaCPPBridge(direction);
	if (Jthis==-1) throw new Exception("WireNative::WireNative(): Could not create new WireNative object.");
//        System.out.println("WireNative: Konstruktor Ende Jthis "+Jthis);
	}
        
private native int newJavaCPPBridge(int devicenumber);

public synchronized void finalize()
	{
  //          System.out.println("WireNative: finalize Jthis"+Jthis);
	try{Jclose(Jthis);}catch(Exception e) {e.printStackTrace();}

        try{Jfinalize();}catch(Exception e){e.printStackTrace();} // clean up C++ if possible
//            System.out.println("WireNative: finalize End");
	}
private native void Jfinalize();

public synchronized String devicename(int port) throws Exception
	{return Jdevicename(Jthis,port);}

public long getPosition()
    {
    return (JgetPosition(Jthis)*1000);  // da die Schnittstelle die Position in Millisekunden bergibt
    }
        
private native String Jdevicename(int Jthis,int port);

public synchronized void close() throws Exception

	{
//            System.out.println("WireNative: close Jthis "+Jthis);
            Jclose(Jthis);
//            System.out.println("WireNative: close End");
        }
private native void Jclose(int Jthis);

public synchronized void open() throws MidiUnavailableException
	{
//System.out.println("WireNative open "+port+" Jthis:"+Jthis);
//        if (dir==1)
          {
          if (port!=-1) Jopen(Jthis); else throw new MidiUnavailableException("open(): first set the port!");
    }
	}
private native void Jopen(int Jthis);

public synchronized void setport(int port) throws Exception
	{
//System.out.println("WireNative setport "+port);
	if (port!=-1) Jsetport(Jthis,port);// indien de poort niet 'none' is.
	else close();
	this.port=port;
	}
private native void Jsetport(int Jthis,int port);

public synchronized  int numberofdevices()
    {
    return Jnumberofdevices(Jthis);
    }
private native int Jnumberofdevices(int Jthis);

public synchronized int getport() {return port;}

public synchronized int readLine(byte[] inhere) throws Exception
    {
    return JreadLine(inhere,Jthis);
    }

private native int JreadLine(byte[] inhere,int Jthis); // not synchronized because of speed

public synchronized void writeLine(byte[] ainarray,int who) throws Exception
	{JwriteLine(Jthis,ainarray,who);}

private native void JwriteLine(int Jthis,byte[] outhere,int who); // not synchronized because of speed

private native long JgetPosition(int Jthis); // returns the actual position in Microseconds


}


