/*
 *  Copyright (c) 1999 by Niels Gorisse <niels@bonneville.nl>
 *  Modifications for WireProvider and Large Sysex (c) 2000
 *  by Gerrit Gehnen <Gerrit.Gehnen@gmx.de>
 *  Modifications for handling Sysex Continuation (c) 2001
 *  by Brian Klock <jsynthlib@overwhelmed.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
package WireProvider;
import javax.sound.midi.*;
import java.util.Vector;
import java.io.OutputStream;

/**
 * WireOutput represents a MIDI output device.
 * When an exception is thrown, the device is stopped automatically.
 * @author    <A HREF="mailto:niels@bonneville.nl">Niels Gorisse</A>, 22 april 1999, Bonneville
 * @version   WireOutput 0.97
 **/

public final class WireOutput implements MidiDevice
	{
  WireNative myOutput;
  WireDeviceInfo winfo=new WireDeviceInfo ("WireReceiver","Gerrit Gehnen","WireReceiver","0.98");
  private byte[] array;
  private boolean isopen=false;
  
  private Vector receivers;
// Following methods are from interface MidiDevice

class WireOutputReceiver implements Receiver  
  {
    private boolean isopen;
    
    WireOutputReceiver()
    {
//    System.out.println("WireOutputReceiver: Konstruktor");
    isopen=true;
    }
  
  /** Closes the Receiver
  *
  **/
    
  public void close()
    {
//                System.out.println("WireOutputReceiver: close");
    isopen=false;
    receivers.remove(this);
    if (receivers.isEmpty())
      {
      WireOutput.this.close();
      }
    }
  
  /**
     * @param m
     * @param time
     */
    public void send (MidiMessage m,long time)
    {
//      System.out.println ("WireOutputReceiver: send");
      if (isopen)
      {
        try
        {
          //            synchronized
          {
            sendThrow (m,time);
          }
        }
        catch (Exception e2){ System.out.println (e2.toString ());}
      }
    }
}
  
/**
 * @throws Exception
 */
public WireOutput () throws Exception
{
  myOutput = new WireNative (1);
  // preparing output:

  receivers=new Vector ();

  array = new byte[13];

  array[0]=60; // 0: dit is midi
  array[1]=63;
  array[2]=0;
  array[3]=12;
  array[4]=0;
}

/**
 * Close this device. Closing never throws an exception, because when any exception occurs,
 * the device is closed anyway to prevent that the device is located even after the program
 * has quit.
 **/

public void close()
	{
          isopen=false;
//                        System.out.println("WireOutput: close");
        for (int i=0;i<receivers.size ();i++)
            {
            ((WireOutputReceiver)receivers.elementAt (i)).close();
          }
          receivers.removeAllElements ();
//	System.out.println("WireOutput: Vor myoutput.close");
        try{myOutput.close();}catch(Exception e) {;}
//	                        System.out.println("WireOutput: close End");
      }


      /**
       * @return
       */
      public MidiDevice.Info getDeviceInfo ()
      {
        return winfo;
      }


      /** Returns weither this device is open or not. Currently not implemented.
       * @return
       */

public boolean isOpen()
	{ return isopen;}

        /** Open this device.
         * @throws MidiUnavailableException
         */

public void open() throws MidiUnavailableException
	{
	if (myOutput.getport()==-1) return;
	myOutput.open();
        isopen=true;
	}


        /** Get how many receivers are present in this system.
         * @return
         */

public int getMaxReceivers() 
	{
          return -1;
}

/** Set which port this object represents.
 * @param port
 * @throws Exception
 */

public void setport(int port) throws Exception 
{
  myOutput.setport(port);
  winfo=new WireDeviceInfo("Wire: "+myOutput.devicename(port),"Wire Provider","Wire Receiver","0.1");
}

/** Return which port this device represents. -1 is none (default).
 * @return
 */

public int getport() {return myOutput.getport();}

// NU DE RECEIVER IMPLEMENTATIE:



/** This is send() with throwing possible Exceptions.
 * @param event
 * @param time
 * @throws Exception
 */

public void sendThrow(MidiMessage event,long time) throws Exception
	{
	if (event instanceof ShortMessage)
		{
//                System.out.println("Out short message");
		ShortMessage e =(ShortMessage)event;
                array[5]=(byte)event.getStatus();
                array[6]=(byte)e.getData1();
                array[7]=(byte)e.getData2();
                array[8]=0;
                array[9]=(byte)(time&0xff);
                array[10]=(byte)((time>>8)&0xff);
                array[11]=(byte)((time>>16)&0xff);
                array[12]=(byte)((time>>24)&0xff);
		myOutput.writeLine(array,0); // can throw!!
		}
	else if (event instanceof SysexMessage)
		{
		SysexMessage e = (SysexMessage)event;
		byte arree[] = e.getMessage();
//                System.out.println("Out sysex message length"+arree.length);
//                System.out.println("Message:"+arree[0]+" "+arree[1]+" "+arree[2]+" "+arree[3]+" "+(arree[4]&0xff));
/// This code modified by Brian Klock April 2001 to handle sysex continuation
                byte []array2;
                if (e.getStatus()==0xF7)
                  array2 = new byte[arree.length+4]; // 1e: MIDI, 2e: 64/length, 3e: statusbyte               
                  else array2 = new byte[arree.length+5]; // 1e: MIDI, 2e: 64/length, 3e: statusbyte
		array2[0]=60; // dit is midi        	
	        array2[1]=64;
                array2[2]=0; // Reserviert
                array2[3]=(byte)(array2.length&0xff);
                array2[4]=(byte)(array2.length/0x100);

                if (e.getStatus()==0xF7)
                   {
                    for (int q=1;q<arree.length;q++)
                       array2[q+4]=arree[q];
                   } // statusbyte
                else for (int q=0;q<arree.length;q++) array2[q+5]=arree[q];
		myOutput.writeLine(array2,0); // can throw!!
/// End modification
		}
	else throw new Exception("WireOut::send(): Sorry, only ShortEvent and SysexMessage types are supported!");
	}


        /** Return the current timestamp. This method is not in use for
         * MIDI output, so it returns -1. Wire always plays Messages
         * at once without any scheduling at the output.
         * @return
         */

public long getMicrosecondPosition() {return myOutput.getPosition();}




/**
 * @throws MidiUnavailableException
 * @return
 */
public Receiver getReceiver () throws MidiUnavailableException
{
  if (!isOpen ())
  open ();
  WireOutputReceiver wor=new WireOutputReceiver ();
  receivers.addElement (wor);
  return wor;
}

  // unnecessary:

/** This is required by Interface MidiDevice, but there are no Transmitters here
 * @return always zero
 */
public int getMaxTransmitters ()
{return 0;}
  
/** Because there are no Transmitters here, an Exception is always thrown
 * @throws MidiUnavailableException always
 * @return nothing
 */
public Transmitter getTransmitter () throws MidiUnavailableException
{
  throw new MidiUnavailableException ("Sorry, WireOutput has no transmitters");

}

} // end of WireOutput
